@@ -119,6 +119,29 @@ typedef enum /*< prefix=XFDASHBOARD_FIT_MODE >*/
 	XFDASHBOARD_FIT_MODE_BOTH
 } XfdashboardFitMode;
 
+/* Stage background types */
+typedef enum /*< prefix=XFDASHBOARD_STAGE_BACKGROUND_IMAGE_TYPE >*/
+{
+	XFDASHBOARD_STAGE_BACKGROUND_IMAGE_TYPE_NONE=0,
+	XFDASHBOARD_STAGE_BACKGROUND_IMAGE_TYPE_DESKTOP
+} XfdashboardStageBackgroundImageType;
+
+/* Selection target types */
+typedef enum /*< prefix=XFDASHBOARD_SELECTION_TARGET >*/
+{
+	XFDASHBOARD_SELECTION_TARGET_NONE=0, /* Used internally */
+
+	XFDASHBOARD_SELECTION_TARGET_LEFT,
+	XFDASHBOARD_SELECTION_TARGET_RIGHT,
+	XFDASHBOARD_SELECTION_TARGET_UP,
+	XFDASHBOARD_SELECTION_TARGET_DOWN,
+
+	XFDASHBOARD_SELECTION_TARGET_FIRST,
+	XFDASHBOARD_SELECTION_TARGET_LAST,
+
+	XFDASHBOARD_SELECTION_TARGET_NEXT
+} XfdashboardSelectionTarget;
+
 G_END_DECLS
 
 #endif	/* __XFDASHBOARD_TYPES__ */
